@@ -13,63 +13,69 @@
 # You should have received a copy of the GNU General Public License along
 # with this program.  If not, see <http://www.gnu.org/licenses/>.
 ### END LICENSE
-import os
 import hashlib
-from configobj import ConfigObj
-from configobj import DuplicateError
+import logging
+import os
+
+from configobj import ConfigObj, DuplicateError
+from variety.profile import get_profile_path
 from variety.Util import Util
 from variety_lib import varietyconfig
 
-import logging
-
-logger = logging.getLogger('variety')
+logger = logging.getLogger("variety")
 
 TRUTH_VALUES = ["enabled", "1", "true", "on", "yes"]
 
+
 class Options:
-    OUTDATED_HASHES = {'clock_filter': ['dca6bd2dfa2b8c4e2db8801e39208f7f']}
+    OUTDATED_HASHES = {"clock_filter": ["dca6bd2dfa2b8c4e2db8801e39208f7f"]}
+    SIMPLE_DOWNLOADERS = []  # set by VarietyWindow at start
+    IMAGE_SOURCES = []  # set by VarietyWindow at start
+    CONFIGURABLE_IMAGE_SOURCES = []  # set by VarietyWindow at start
+    CONFIGURABLE_IMAGE_SOURCES_MAP = {}  # set by VarietyWindow at start
 
     class SourceType:
-        IMAGE = 1
-        FOLDER = 2
-        FAVORITES = 3
-        FETCHED = 4
-        DESKTOPPR = 6
-        FLICKR = 7
-        APOD = 8
-        MEDIA_RSS = 10
-        EARTH = 11
-        WALLHAVEN = 13
-        REDDIT = 14
-        BING = 15
-        UNSPLASH = 16
-        type_to_str = {
-            FAVORITES: "favorites",
-            FETCHED: "fetched",
-            IMAGE: "image",
-            FOLDER: "folder",
-            DESKTOPPR: "desktoppr",
-            FLICKR: "flickr",
-            APOD: "apod",
-            MEDIA_RSS: "mediarss",
-            EARTH: "earth",
-            WALLHAVEN: "wallhaven",
-            REDDIT: "reddit",
-            BING: "bing",
-            UNSPLASH: "unsplash",
+        # local files and folders
+        IMAGE = "image"
+        FOLDER = "folder"
+        ALBUM_FILENAME = "album (by filename)"
+        ALBUM_DATE = "album (by date)"
+
+        # special local folders
+        FAVORITES = "favorites"
+        FETCHED = "fetched"
+
+        # predefined configurable sources
+        FLICKR = "flickr"
+
+        WALLHAVEN = "wallhaven"
+
+        BUILTIN_SOURCE_TYPES = {
+            IMAGE,
+            FOLDER,
+            ALBUM_FILENAME,
+            ALBUM_DATE,
+            FAVORITES,
+            FETCHED,
+            FLICKR,
         }
 
-        str_to_type = {v: k for k, v in type_to_str.items()}
+        LOCAL_PATH_TYPES = {IMAGE, FOLDER, ALBUM_FILENAME, ALBUM_DATE}
 
-        dl_types = [DESKTOPPR, FLICKR, APOD, MEDIA_RSS, EARTH,
-                    WALLHAVEN, REDDIT, BING, UNSPLASH]
+        LOCAL_TYPES = {IMAGE, FOLDER, ALBUM_FILENAME, ALBUM_DATE, FAVORITES, FETCHED}
+
+        DL_TYPES = {FLICKR}
+
+        EDITABLE_DL_TYPES = {FLICKR}
+
+        REMOVABLE_TYPES = {FOLDER, IMAGE, ALBUM_FILENAME, ALBUM_DATE} | EDITABLE_DL_TYPES
 
     class LightnessMode:
         DARK = 0
         LIGHT = 1
 
     def __init__(self):
-        self.configfile = os.path.expanduser("~/.config/variety/variety.conf")
+        self.configfile = os.path.join(get_profile_path(), "variety.conf")
 
     def read(self):
         self.set_defaults()
@@ -95,20 +101,23 @@ class Options:
             except Exception:
                 pass
 
+            try:
+                self.internet_enabled = config["internet_enabled"].lower() in TRUTH_VALUES
+            except Exception:
+                pass
+
             try:
                 self.safe_mode = config["safe_mode"].lower() in TRUTH_VALUES
             except Exception:
                 pass
 
             try:
-                self.download_enabled = config["download_enabled"].lower() in TRUTH_VALUES
+                self.set_wallpaper_script = os.path.expanduser(config["set_wallpaper_script"])
             except Exception:
                 pass
 
             try:
-                self.download_interval = int(config["download_interval"])
-                if self.download_interval < 60:
-                    self.download_interval = 60
+                self.get_wallpaper_script = os.path.expanduser(config["get_wallpaper_script"])
             except Exception:
                 pass
 
@@ -117,15 +126,25 @@ class Options:
             except Exception:
                 pass
 
+            try:
+                self.download_preference_ratio = max(
+                    0, min(1, float(config["download_preference_ratio"]))
+                )
+            except Exception:
+                pass
+
             try:
                 self.quota_enabled = config["quota_enabled"].lower() in TRUTH_VALUES
             except Exception:
                 pass
 
             try:
-                self.quota_size = int(config["quota_size"])
-                if self.quota_size < 50:
-                    self.quota_size = 50
+                self.quota_size = max(50, int(config["quota_size"]))
+            except Exception:
+                pass
+
+            try:
+                self.wallhaven_api_key = str(config["wallhaven_api_key"]).strip()
             except Exception:
                 pass
 
@@ -136,7 +155,19 @@ class Options:
 
             try:
                 favorites_ops_text = config["favorites_operations"]
-                self.favorites_operations = list([x.strip().split(':') for x in favorites_ops_text.split(';') if x])
+                self.favorites_operations = list(
+                    [x.strip().split(":") for x in favorites_ops_text.split(";") if x]
+                )
+            except Exception:
+                pass
+
+            try:
+                self.wallpaper_auto_rotate = config["wallpaper_auto_rotate"].lower() in TRUTH_VALUES
+            except Exception:
+                pass
+
+            try:
+                self.wallpaper_display_mode = str(config["wallpaper_display_mode"]).strip()
             except Exception:
                 pass
 
@@ -151,18 +182,22 @@ class Options:
                 pass
 
             try:
-                self.clipboard_use_whitelist = config["clipboard_use_whitelist"].lower() in TRUTH_VALUES
+                self.clipboard_use_whitelist = (
+                    config["clipboard_use_whitelist"].lower() in TRUTH_VALUES
+                )
             except Exception:
                 pass
 
             try:
-                self.clipboard_hosts = config["clipboard_hosts"].lower().split(',')
+                self.clipboard_hosts = config["clipboard_hosts"].lower().split(",")
             except Exception:
                 pass
 
             try:
                 icon = config["icon"]
-                if icon in ["Light", "Dark", "Current", "None"] or (os.access(icon, os.R_OK) and Util.is_image(icon)):
+                if icon in ["Light", "Dark", "Current", "1", "2", "3", "4", "None"] or (
+                    os.access(icon, os.R_OK) and Util.is_image(icon)
+                ):
                     self.icon = icon
             except Exception:
                 pass
@@ -320,7 +355,7 @@ class Options:
                 pass
 
             try:
-                self.quotes_disabled_sources = config["quotes_disabled_sources"].strip().split('|')
+                self.quotes_disabled_sources = config["quotes_disabled_sources"].strip().split("|")
             except Exception:
                 pass
 
@@ -364,34 +399,48 @@ class Options:
             except Exception:
                 pass
 
+            try:
+                self.quotes_max_length = int(config["quotes_max_length"])
+                self.quotes_max_length = max(0, self.quotes_max_length)
+            except Exception:
+                pass
+
             try:
                 self.quotes_favorites_file = os.path.expanduser(config["quotes_favorites_file"])
             except Exception:
                 pass
 
             try:
-                self.slideshow_sources_enabled = config["slideshow_sources_enabled"].lower() in TRUTH_VALUES
+                self.slideshow_sources_enabled = (
+                    config["slideshow_sources_enabled"].lower() in TRUTH_VALUES
+                )
             except Exception:
                 pass
 
             try:
-                self.slideshow_favorites_enabled = config["slideshow_favorites_enabled"].lower() in TRUTH_VALUES
+                self.slideshow_favorites_enabled = (
+                    config["slideshow_favorites_enabled"].lower() in TRUTH_VALUES
+                )
             except Exception:
                 pass
 
             try:
-                self.slideshow_downloads_enabled = config["slideshow_downloads_enabled"].lower() in TRUTH_VALUES
+                self.slideshow_downloads_enabled = (
+                    config["slideshow_downloads_enabled"].lower() in TRUTH_VALUES
+                )
             except Exception:
                 pass
 
             try:
-                self.slideshow_custom_enabled = config["slideshow_custom_enabled"].lower() in TRUTH_VALUES
+                self.slideshow_custom_enabled = (
+                    config["slideshow_custom_enabled"].lower() in TRUTH_VALUES
+                )
             except Exception:
                 pass
 
             try:
                 custom_path = config["slideshow_custom_folder"]
-                if custom_path in ('None', 'Default') or not os.path.isdir(custom_path):
+                if custom_path in ("None", "Default") or not os.path.isdir(custom_path):
                     self.slideshow_custom_folder = Util.get_xdg_pictures_folder()
                 else:
                     self.slideshow_custom_folder = custom_path
@@ -400,7 +449,13 @@ class Options:
 
             try:
                 slideshow_sort_order = config["slideshow_sort_order"]
-                if slideshow_sort_order in ["Random", "Name, asc", "Name, desc", "Date, asc", "Date, desc"]:
+                if slideshow_sort_order in [
+                    "Random",
+                    "Name, asc",
+                    "Name, desc",
+                    "Date, asc",
+                    "Date, desc",
+                ]:
                     self.slideshow_sort_order = slideshow_sort_order
             except Exception:
                 pass
@@ -441,15 +496,23 @@ class Options:
             except Exception:
                 pass
 
+            self.sources = []
             if "sources" in config:
-                self.sources = []
                 sources = config["sources"]
                 for v in sources.values():
                     try:
                         self.sources.append(Options.parse_source(v))
                     except Exception:
                         logger.debug(lambda: "Cannot parse source: " + v, exc_info=True)
-                        logger.info('Ignoring no longer supported source %s', v)
+                        logger.info("Ignoring no longer supported source %s", v)
+
+            # automatically append sources for all simple downloaders we have
+            source_types = set(s[1] for s in self.sources)
+            for downloader in sorted(self.SIMPLE_DOWNLOADERS, key=lambda dl: dl.get_source_type()):
+                if downloader.get_source_type() not in source_types:
+                    self.sources.append(
+                        [False, downloader.get_source_type(), downloader.get_description()]
+                    )
 
             self.parse_autosources()
 
@@ -478,16 +541,18 @@ class Options:
                 current_hash = hashlib.md5(config[key].encode()).hexdigest()
                 if current_hash in outdated_hashes:
                     # entry is outdated: delete it and use the default
-                    logger.warning(lambda: "Option " + key + " has an outdated value, using the new default")
+                    logger.warning(
+                        lambda: "Option " + key + " has an outdated value, using the new default"
+                    )
                     changed = True
                     del config[key]
         return changed
 
     def parse_autosources(self):
         try:
-            with open(varietyconfig.get_data_file("config", "sources.txt"), encoding='utf8') as f:
+            with open(varietyconfig.get_data_file("config", "sources.txt"), encoding="utf8") as f:
                 for line in f:
-                    if not line.strip() or line.strip().startswith('#'):
+                    if not line.strip() or line.strip().startswith("#"):
                         continue
                     try:
                         s = Options.parse_source(line.strip())
@@ -501,13 +566,18 @@ class Options:
 
     def parse_autofilters(self):
         try:
-            with open(varietyconfig.get_data_file("config", "filters.txt"), encoding='utf8') as f:
+            with open(varietyconfig.get_data_file("config", "filters.txt"), encoding="utf8") as f:
                 for line in f:
-                    if not line.strip() or line.strip().startswith('#'):
+                    if not line.strip() or line.strip().startswith("#"):
                         continue
                     try:
                         s = Options.parse_filter(line.strip())
-                        if not s[1].lower() in [f[1].lower() for f in self.filters]:
+                        for f in self.filters:
+                            if f[1].lower() == s[1].lower():
+                                f[2] = s[2]
+                                break
+                        else:
+                            # not found at all in filters, append it
                             self.filters.append(s)
                     except Exception:
                         logger.exception(lambda: "Cannot parse filter in filters.txt: " + line)
@@ -516,47 +586,72 @@ class Options:
 
     @staticmethod
     def parse_source(v):
-        s = v.strip().split('|')
-        enabled = s[0].lower() in TRUTH_VALUES
-        return [enabled, (Options.str_to_type(s[1])), s[2]]
-
-    @staticmethod
-    def parse_filter(v):
-        s = v.strip().split('|')
+        s = v.strip().split("|")
         enabled = s[0].lower() in TRUTH_VALUES
         return [enabled, s[1], s[2]]
 
     @staticmethod
-    def str_to_type(s):
-        s = s.lower()
-        if s in Options.SourceType.str_to_type:
-            return Options.SourceType.str_to_type[s]
-        else:
-            raise Exception("Unknown source type")
+    def parse_filter(v):
+        s = v.strip().split("|")
+        enabled = s[0].lower() in TRUTH_VALUES
+        return [enabled, s[1], s[2]]
 
     @staticmethod
-    def type_to_str(stype):
-        return Options.SourceType.type_to_str[stype]
+    def get_all_supported_source_types():
+        return Options.SourceType.BUILTIN_SOURCE_TYPES | Options.get_plugin_source_types()
+
+    @staticmethod
+    def get_downloader_source_types():
+        return Options.SourceType.DL_TYPES | Options.get_plugin_source_types()
+
+    @staticmethod
+    def get_editable_source_types():
+        return Options.SourceType.EDITABLE_DL_TYPES | Options.get_configurable_plugin_source_types()
+
+    @staticmethod
+    def get_removable_source_types():
+        return Options.SourceType.REMOVABLE_TYPES | Options.get_editable_source_types()
+
+    @staticmethod
+    def get_plugin_source_types():
+        return set(dl.get_source_type() for dl in Options.IMAGE_SOURCES)
+
+    @staticmethod
+    def get_configurable_plugin_source_types():
+        return set(dl.get_source_type() for dl in Options.CONFIGURABLE_IMAGE_SOURCES)
 
     def set_defaults(self):
         self.change_enabled = True
         self.change_on_start = False
         self.change_interval = 300
+        self.internet_enabled = True
         self.safe_mode = False
 
-        self.download_enabled = True
-        self.download_interval = 600
-        self.download_folder = os.path.expanduser("~/.config/variety/Downloaded")
+        self.set_wallpaper_script = os.path.join(get_profile_path(), "scripts", "set_wallpaper")
+        self.get_wallpaper_script = os.path.join(get_profile_path(), "scripts", "get_wallpaper")
+
+        self.download_folder = os.path.join(get_profile_path(), "Downloaded")
+        self.download_preference_ratio = 0.9
         self.quota_enabled = True
-        self.quota_size = 500
+        self.quota_size = 1000
+        self.wallhaven_api_key = ""
 
-        self.favorites_folder = os.path.expanduser("~/.config/variety/Favorites")
-        self.favorites_operations = [["Downloaded", "Copy"], ["Fetched", "Move"], ["Others", "Copy"]]
+        self.favorites_folder = os.path.join(get_profile_path(), "Favorites")
+        self.favorites_operations = [
+            ["Downloaded", "Copy"],
+            ["Fetched", "Move"],
+            ["Others", "Copy"],
+        ]
 
-        self.fetched_folder = os.path.expanduser("~/.config/variety/Fetched")
+        self.wallpaper_auto_rotate = True
+        self.wallpaper_display_mode = "os"
+
+        self.fetched_folder = os.path.join(get_profile_path(), "Fetched")
         self.clipboard_enabled = False
         self.clipboard_use_whitelist = True
-        self.clipboard_hosts = "alpha.wallhaven.cc,ns223506.ovh.net,wallpapers.net,flickr.com,imgur.com,deviantart.com,interfacelift.com,vladstudio.com".split(',')
+        self.clipboard_hosts = "wallhaven.cc,ns223506.ovh.net,wallpapers.net,flickr.com,imgur.com,deviantart.com,interfacelift.com,vladstudio.com".split(
+            ","
+        )
 
         self.icon = "Light"
 
@@ -582,12 +677,12 @@ class Options:
         self.copyto_folder = "Default"
 
         self.clock_enabled = False
-        self.clock_font = "Ubuntu Condensed, 70"
-        self.clock_date_font = "Ubuntu Condensed, 30"
+        self.clock_font = "Serif 70"
+        self.clock_date_font = "Serif 30"
         self.clock_filter = "-density 100 -font `fc-match -f '%{file[0]}' '%CLOCK_FONT_NAME'` -pointsize %CLOCK_FONT_SIZE -gravity SouthEast -fill '#00000044' -annotate 0x0+[%HOFFSET+58]+[%VOFFSET+108] '%H:%M' -fill white -annotate 0x0+[%HOFFSET+60]+[%VOFFSET+110] '%H:%M' -font `fc-match -f '%{file[0]}' '%DATE_FONT_NAME'` -pointsize %DATE_FONT_SIZE -fill '#00000044' -annotate 0x0+[%HOFFSET+58]+[%VOFFSET+58] '%A, %B %d' -fill white -annotate 0x0+[%HOFFSET+60]+[%VOFFSET+60] '%A, %B %d'"
 
         self.quotes_enabled = False
-        self.quotes_font = "Bitstream Charter 30"
+        self.quotes_font = "Serif 30"
         self.quotes_text_color = (255, 255, 255)
         self.quotes_bg_color = (80, 80, 80)
         self.quotes_bg_opacity = 55
@@ -600,7 +695,8 @@ class Options:
         self.quotes_width = 70
         self.quotes_hpos = 100
         self.quotes_vpos = 40
-        self.quotes_favorites_file = os.path.expanduser("~/.config/variety/favorite_quotes.txt")
+        self.quotes_max_length = 250
+        self.quotes_favorites_file = os.path.join(get_profile_path(), "favorite_quotes.txt")
 
         self.slideshow_sources_enabled = True
         self.slideshow_favorites_enabled = True
@@ -619,11 +715,11 @@ class Options:
             [True, Options.SourceType.FAVORITES, "The Favorites folder"],
             [True, Options.SourceType.FETCHED, "The Fetched folder"],
             [True, Options.SourceType.FOLDER, "/usr/share/backgrounds/"],
-            [True, Options.SourceType.DESKTOPPR, "Random wallpapers from Desktoppr.co"],
-            [True, Options.SourceType.BING, "Bing Photo of the Day"],
-            [True, Options.SourceType.UNSPLASH, "High-resolution photos from Unsplash.com"],
-            [False, Options.SourceType.APOD, "NASA's Astronomy Picture of the Day"],
-            [True, Options.SourceType.FLICKR, "user:www.flickr.com/photos/peter-levi/;user_id:93647178@N00;"],
+            [
+                True,
+                Options.SourceType.FLICKR,
+                "user:www.flickr.com/photos/peter-levi/;user_id:93647178@N00;",
+            ],
         ]
 
         self.filters = [
@@ -632,43 +728,54 @@ class Options:
             [False, "Heavy blur", "-blur 120x40"],
             [False, "Oil painting", "-paint 6"],
             [False, "Charcoal painting", "-charcoal 3"],
-            [False, "Pencil sketch", """-colorspace gray \( +clone -tile ~/.config/variety/pencil_tile.png -draw "color 0,0 reset" +clone +swap -compose color_dodge -composite \) -fx 'u*.2+v*.8'"""],
             [False, "Pointilism", "-spread 10 -noise 3"],
-            [False, "Pixellate", "-scale 3% -scale 3333%"]
+            [False, "Pixellate", "-scale 3% -scale 3333%"],
         ]
 
     def write(self):
         try:
-            config = ConfigObj(self.configfile, encoding='utf8', default_encoding='utf8')
+            config = ConfigObj(self.configfile, encoding="utf8", default_encoding="utf8")
         except Exception:
-            config = ConfigObj(encoding='utf8', default_encoding='utf8')
+            config = ConfigObj(encoding="utf8", default_encoding="utf8")
             config.filename = self.configfile
 
         try:
             config["change_enabled"] = str(self.change_enabled)
             config["change_on_start"] = str(self.change_on_start)
             config["change_interval"] = str(self.change_interval)
+            config["internet_enabled"] = str(self.internet_enabled)
             config["safe_mode"] = str(self.safe_mode)
 
-            config["download_enabled"] = str(self.download_enabled)
-            config["download_interval"] = str(self.download_interval)
+            config["set_wallpaper_script"] = Util.collapseuser(self.set_wallpaper_script)
+            config["get_wallpaper_script"] = Util.collapseuser(self.get_wallpaper_script)
+
             config["download_folder"] = Util.collapseuser(self.download_folder)
+            config["download_preference_ratio"] = str(self.download_preference_ratio)
 
             config["quota_enabled"] = str(self.quota_enabled)
             config["quota_size"] = str(self.quota_size)
 
+            config["wallhaven_api_key"] = str(self.wallhaven_api_key)
+
             config["favorites_folder"] = Util.collapseuser(self.favorites_folder)
-            config["favorites_operations"] = ';'.join(':'.join(x) for x in self.favorites_operations)
+            config["favorites_operations"] = ";".join(
+                ":".join(x) for x in self.favorites_operations
+            )
+
+            config["wallpaper_auto_rotate"] = str(self.wallpaper_auto_rotate)
+            config["wallpaper_display_mode"] = str(self.wallpaper_display_mode)
 
             config["fetched_folder"] = Util.collapseuser(self.fetched_folder)
             config["clipboard_enabled"] = str(self.clipboard_enabled)
             config["clipboard_use_whitelist"] = str(self.clipboard_use_whitelist)
-            config["clipboard_hosts"] = ','.join(self.clipboard_hosts)
+            config["clipboard_hosts"] = ",".join(self.clipboard_hosts)
 
             config["icon"] = self.icon
 
             config["desired_color_enabled"] = str(self.desired_color_enabled)
-            config["desired_color"] = " ".join(map(str, self.desired_color)) if self.desired_color else "None"
+            config["desired_color"] = (
+                " ".join(map(str, self.desired_color)) if self.desired_color else "None"
+            )
             config["min_size_enabled"] = str(self.min_size_enabled)
             config["min_size"] = str(self.min_size)
             config["use_landscape_enabled"] = str(self.use_landscape_enabled)
@@ -699,7 +806,7 @@ class Options:
             config["quotes_bg_color"] = " ".join(map(str, self.quotes_bg_color))
             config["quotes_bg_opacity"] = str(self.quotes_bg_opacity)
             config["quotes_text_shadow"] = str(self.quotes_text_shadow)
-            config["quotes_disabled_sources"] = '|'.join(self.quotes_disabled_sources)
+            config["quotes_disabled_sources"] = "|".join(self.quotes_disabled_sources)
             config["quotes_tags"] = self.quotes_tags
             config["quotes_authors"] = self.quotes_authors
             config["quotes_change_enabled"] = str(self.quotes_change_enabled)
@@ -707,6 +814,7 @@ class Options:
             config["quotes_width"] = str(self.quotes_width)
             config["quotes_hpos"] = str(self.quotes_hpos)
             config["quotes_vpos"] = str(self.quotes_vpos)
+            config["quotes_max_length"] = str(self.quotes_max_length)
             config["quotes_favorites_file"] = Util.collapseuser(self.quotes_favorites_file)
 
             config["slideshow_sources_enabled"] = str(self.slideshow_sources_enabled)
@@ -724,7 +832,7 @@ class Options:
 
             config["sources"] = {}
             for i, s in enumerate(self.sources):
-                config["sources"]["src" + str(i + 1)] = str(s[0]) + "|" + str(Options.type_to_str(s[1])) + "|" + s[2]
+                config["sources"]["src" + str(i + 1)] = str(s[0]) + "|" + str(s[1]) + "|" + s[2]
 
             config["filters"] = {}
             for i, f in enumerate(self.filters):
@@ -743,7 +851,7 @@ class Options:
         config.write()
 
     def read_config(self):
-        config = ConfigObj(raise_errors=False, encoding='utf8', default_encoding='utf8')
+        config = ConfigObj(raise_errors=False, encoding="utf8", default_encoding="utf8")
         config.filename = self.configfile
         try:
             config.reload()
@@ -751,10 +859,11 @@ class Options:
             logger.warning(lambda: "Duplicate keys in config file, please fix this")
         return config
 
+
 if __name__ == "__main__":
     formatter = logging.Formatter("%(levelname)s:%(name)s: %(funcName)s() '%(message)s'")
 
-    logger = logging.getLogger('variety')
+    logger = logging.getLogger("variety")
     logger_sh = logging.StreamHandler()
     logger_sh.setFormatter(formatter)
     logger.addHandler(logger_sh)
