@@ -317,7 +317,13 @@ class ModuleProfiler:
             )
 
 
+class InternetDisabledError(Exception):
+    pass
+
+
 class Util:
+    internet_enabled = True
+
     @staticmethod
     def sanitize_filename(filename):
         valid_chars = " ,.!-+@()_%s%s" % (string.ascii_letters, string.digits)
@@ -391,6 +397,9 @@ class Util:
     def list_files(
         files=(), folders=(), filter_func=(lambda f: True), max_files=10000, randomize=True
     ):
+        class NextFolderException(Exception):
+            pass
+
         count = 0
         for filepath in files:
             logger.debug(
@@ -405,29 +414,38 @@ class Util:
             random.shuffle(folders)
 
         for folder in folders:
-            if os.path.isdir(folder):
-                try:
-                    for root, subFolders, files in os.walk(folder, followlinks=True):
-                        if randomize:
-                            random.shuffle(files)
-                            random.shuffle(subFolders)
-                        for filename in files:
-                            logger.debug(
-                                lambda: "checking file %s against filter_func %s (root=%s)"
-                                % (filename, filter_func, root)
-                            )
-                            path = os.path.join(root, filename)
-                            if filter_func(path):
-                                count += 1
-                                if count > max_files:
-                                    logger.info(
-                                        lambda: "More than %d files in the folders, stop listing"
-                                        % max_files
-                                    )
-                                    return
-                                yield path
-                except Exception:
-                    logger.exception(lambda: "Could not walk folder " + folder)
+            folder_quota = max(20, int(max_files / len(folders)))
+            if not os.path.isdir(folder):
+                continue
+            try:
+                count_in_folder = 0
+                for root, subfolders, files in os.walk(folder, followlinks=True):
+                    subfolder_quota = max(10, int(folder_quota / (1 + len(subfolders))))
+                    if randomize:
+                        random.shuffle(files)
+                        random.shuffle(subfolders)
+                    for filename in files[:subfolder_quota]:
+                        logger.debug(
+                            lambda: "checking file %s against filter_func %s (root=%s)"
+                            % (filename, filter_func, root)
+                        )
+                        path = os.path.join(root, filename)
+                        if filter_func(path):
+                            count += 1
+                            if count > max_files:
+                                logger.info(
+                                    lambda: "More than %d files in the folders, stop listing"
+                                    % max_files
+                                )
+                                return
+                            yield path
+                            count_in_folder += 1
+                            if count_in_folder > folder_quota:
+                                raise NextFolderException
+            except NextFolderException:
+                continue
+            except Exception:
+                logger.exception(lambda: "Could not walk folder " + folder)
 
     @staticmethod
     def start_force_exit_thread(delay):
@@ -589,6 +607,25 @@ class Util:
         else:
             return image_width, image_height
 
+    @staticmethod
+    def get_primary_display_size(hidpi_scaled=True):
+        display = Gdk.Display.get_default()
+        monitor = display.get_primary_monitor()
+        if not monitor:
+            monitor = display.get_monitor(0)
+
+        if monitor:
+            geometry = monitor.get_geometry()
+            scale = monitor.get_scale_factor() if hidpi_scaled else 1.0
+            return int(geometry.width * scale), int(geometry.height * scale)
+        else:
+            return Util.get_multimonitor_display_size()
+
+    @staticmethod
+    def get_multimonitor_display_size():
+        screen = Gdk.Screen.get_default()
+        return screen.get_width(), screen.get_height()
+
     @staticmethod
     def find_unique_name(filename):
         index = filename.rfind(".")
@@ -605,6 +642,9 @@ class Util:
 
     @staticmethod
     def request(url, data=None, stream=False, method=None, timeout=5, headers=None):
+        if not Util.internet_enabled:
+            raise InternetDisabledError("Internet access in Variety is currently disabled")
+
         if url.startswith("//"):
             url = "http:" + url
         headers = headers or {}
@@ -693,10 +733,7 @@ class Util:
     def get_scaled_size(image):
         """Computes the size to which the image is scaled to fit the screen: original_size * scale_ratio = scaled_size"""
         iw, ih = Util.get_size(image)
-        screen_w, screen_h = (
-            Gdk.Screen.get_default().get_width(),
-            Gdk.Screen.get_default().get_height(),
-        )
+        screen_w, screen_h = Util.get_primary_display_size()
         screen_ratio = float(screen_w) / screen_h
         if (
             screen_ratio > float(iw) / ih
@@ -705,22 +742,6 @@ class Util:
         else:  # image is "wider" than the screen ratio - need to offset horizontally
             return int(round(iw * float(screen_h) / ih)), screen_h
 
-    @staticmethod
-    def get_scale_to_screen_ratio(image):
-        """Computes the ratio by which the image is scaled to fit the screen: original_size * scale_ratio = scaled_size"""
-        iw, ih = Util.get_size(image)
-        screen_w, screen_h = (
-            Gdk.Screen.get_default().get_width(),
-            Gdk.Screen.get_default().get_height(),
-        )
-        screen_ratio = float(screen_w) / screen_h
-        if (
-            screen_ratio > float(iw) / ih
-        ):  # image is "taller" than the screen ratio - need to offset vertically
-            return int(float(screen_w) / iw)
-        else:  # image is "wider" than the screen ratio - need to offset horizontally
-            return int(float(screen_h) / ih)
-
     @staticmethod
     def gtk_to_fcmatch_font(gtk_font_name):
         fd = Pango.FontDescription(gtk_font_name)
@@ -912,10 +933,6 @@ class Util:
                     total_size += os.path.getsize(fp)
         return total_size
 
-    @staticmethod
-    def get_screen_width():
-        return Gdk.Screen.get_default().get_width()
-
 
 def on_gtk(f):
     @functools.wraps(f)
